"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeAPIsMixin = void 0;
const constants_1 = require("@litprotocol-dev/constants");
const misc_1 = require("@litprotocol-dev/misc");
function NodeAPIsMixin(base) {
    return class extends base {
        constructor() {
            super(...arguments);
            // ==================== SENDING COMMAND ====================
            /**
             *
             * Send a command to nodes
             *
             * @param { SendNodeCommand }
             *
             * @returns { Promise<any> }
             *
             */
            this.sendCommandToNode = async ({ url, data, }) => {
                (0, misc_1.log)(`sendCommandToNode with url ${url} and data`, data);
                const req = {
                    method: 'POST',
                    headers: {
                        'Content-Type': 'application/json',
                        'lit-js-sdk-version': constants_1.version,
                    },
                    body: JSON.stringify(data),
                };
                return fetch(url, req).then(async (response) => {
                    const isJson = response.headers
                        .get('content-type')
                        ?.includes('application/json');
                    const data = isJson ? await response.json() : null;
                    if (!response.ok) {
                        // get error message from body or default to response status
                        const error = data || response.status;
                        return Promise.reject(error);
                    }
                    return data;
                });
            };
            // ==================== API Calls to Nodes ====================
            /**
             *
             * Get JS Execution Shares from Nodes
             *
             * @param { JsonExecutionRequest } params
             *
             * @returns { Promise<any> }
             */
            this.getJsExecutionShares = async (url, params) => {
                const { code, ipfsId, authSig, jsParams, sessionSigs } = params;
                (0, misc_1.log)('getJsExecutionShares');
                // -- execute
                const urlWithPath = `${url}/web/execute`;
                const data = {
                    code,
                    ipfsId,
                    authSig,
                    jsParams,
                };
                return await this.sendCommandToNode({ url: urlWithPath, data });
            };
            /**
             *
             * Get Chain Data Signing Shares
             *
             * @param { string } url
             * @param { JsonSignChainDataRequest } params
             *
             * @returns { Promise<any> }
             *
             */
            this.getChainDataSigningShare = async (url, params) => {
                const { callRequests, chain, iat, exp } = params;
                (0, misc_1.log)('getChainDataSigningShare');
                const urlWithPath = `${url}/web/signing/sign_chain_data`;
                const data = {
                    callRequests,
                    chain,
                    iat,
                    exp,
                };
                return await this.sendCommandToNode({ url: urlWithPath, data });
            };
            /**
             *
             * Get Signing Shares from Nodes
             *
             * @param { string } url
             * @param { JsonSigningRetrieveRequest } params
             *
             * @returns { Promise<any>}
             *
             */
            this.getSigningShare = async (url, params) => {
                (0, misc_1.log)('getSigningShare');
                const urlWithPath = `${url}/web/signing/retrieve`;
                return await this.sendCommandToNode({
                    url: urlWithPath,
                    data: params,
                });
            };
            /**
             *
             * Ger Decryption Shares from Nodes
             *
             * @param { string } url
             * @param { JsonEncryptionRetrieveRequest } params
             *
             * @returns { Promise<any> }
             *
             */
            this.getDecryptionShare = async (url, params) => {
                (0, misc_1.log)('getDecryptionShare');
                const urlWithPath = `${url}/web/encryption/retrieve`;
                return await this.sendCommandToNode({
                    url: urlWithPath,
                    data: params,
                });
            };
            /**
             *
             * Store signing conditions to nodes
             *
             * @param { string } url
             * @param { JsonSigningStoreRequest } params
             *
             * @returns { Promise<NodeCommandResponse> }
             *
             */
            this.storeSigningConditionWithNode = async (url, params) => {
                (0, misc_1.log)('storeSigningConditionWithNode');
                const urlWithPath = `${url}/web/signing/store`;
                return await this.sendCommandToNode({
                    url: urlWithPath,
                    data: {
                        key: params.key,
                        val: params.val,
                        authSig: params.authSig,
                        chain: params.chain,
                        permanant: params.permanent,
                    },
                });
            };
            /**
             *
             * Store encryption conditions to nodes
             *
             * @param { string } urk
             * @param { JsonEncryptionStoreRequest } params
             *
             * @returns { Promise<NodeCommandResponse> }
             *
             */
            this.storeEncryptionConditionWithNode = async (url, params) => {
                (0, misc_1.log)('storeEncryptionConditionWithNode');
                const urlWithPath = `${url}/web/encryption/store`;
                const data = {
                    key: params.key,
                    val: params.val,
                    authSig: params.authSig,
                    chain: params.chain,
                    permanant: params.permanent,
                };
                return await this.sendCommandToNode({ url: urlWithPath, data });
            };
            /**
             *
             * Sign wit ECDSA
             *
             * @param { string } url
             * @param { SignWithECDSA } params
             *
             * @returns { Promise}
             *
             */
            this.signECDSA = async (url, params) => {
                console.log('sign_message_ecdsa');
                const urlWithPath = `${url}/web/signing/sign_message_ecdsa`;
                return await this.sendCommandToNode({
                    url: urlWithPath,
                    data: params,
                });
            };
            /**
             *
             * Sign Condition ECDSA
             *
             * @param { string } url
             * @param { SignConditionECDSA } params
             *
             * @returns { Promise<NodeCommandResponse> }
             *
             */
            this.signConditionEcdsa = async (url, params) => {
                (0, misc_1.log)('signConditionEcdsa');
                const urlWithPath = `${url}/web/signing/signConditionEcdsa`;
                const data = {
                    access_control_conditions: params.accessControlConditions,
                    evmContractConditions: params.evmContractConditions,
                    solRpcConditions: params.solRpcConditions,
                    auth_sig: params.auth_sig,
                    chain: params.chain,
                    iat: params.iat,
                    exp: params.exp,
                };
                return await this.sendCommandToNode({
                    url: urlWithPath,
                    data,
                });
            };
            /**
             *
             * Handshake with SGX
             *
             * @param { HandshakeWithSgx } params
             *
             * @returns { Promise<NodeCommandServerKeysResponse> }
             *
             */
            this.handshakeWithSgx = async (params) => {
                // -- get properties from params
                const { url } = params;
                // -- create url with path
                const urlWithPath = `${url}/web/handshake`;
                (0, misc_1.log)(`handshakeWithSgx ${urlWithPath}`);
                const data = {
                    clientPublicKey: 'test',
                };
                return await this.sendCommandToNode({
                    url: urlWithPath,
                    data,
                });
            };
        }
    };
}
exports.NodeAPIsMixin = NodeAPIsMixin;
//# sourceMappingURL=node-apis-mixin.js.map